#ifndef DAQDRIVER_H
#define DAQDRIVER_H

// Sep 23, 2011, V2.0.3 fixed DAQ_ReadCalibrator
// Sep 22, 2011, V2.0.2 added DAQ_ReadCalibrator
// Aug 30, 2011, V2.0.1 created

// The following ifdef block is the standard way of creating macros
// which make exporting from a DLL simpler.  All files within this DLL
// are compiled with the DAQ_EXPORTS symbol defined on the command line.
// This symbol should not be defined on any project that uses this DLL.
// This way any other project whose source files include this file see
// DAQ_API functions as being imported from a DLL, whereas this DLL
// sees symbols defined with this macro as being exported.
#ifdef DAQ_EXPORTS
#define DAQ_API __declspec(dllexport)
#else
#define DAQ_API __declspec(dllimport)
#endif

typedef PVOID	DAQ_HANDLE;
typedef ULONG	DAQ_STATUS;

enum {
	DAQ_OK, // success
	DAQ_OPEN_FAILURE,
	DAQ_WRITE_FAILURE,
	DAQ_READ_FAILURE,
	DAQ_COMM_FAILURE,
	DAQ_TIMEOUT,
	DAQ_INVALID_HANDLE,
	DAQ_INVALID_PARA,
	DAQ_UNDEFINED_RANGE,
	DAQ_UNKNOWN
};

typedef enum DAQ_ADCMODE {
	MODE_DIFFERENTIAL = 1,
	MODE_SINGLEENDED = 2
} DAQ_ADCMODE;

typedef enum DAQ_ADCRANGE {
	RANGE_N10000_P10000 = 1,
	RANGE_N5000_P5000 = 2,
	RANGE_N2500_P2500 = 4,
	RANGE_N1250_P1250 = 8
} DAQ_ADCRANGE;

typedef enum DAQ_DACWAVE {
	SINE_WAVE = 4,
	TRIANGLE_WAVE = 8,
	SAWTOOTH_WAVE = 16
} DAQ_DACWAVE;

#ifdef __cplusplus
extern "C" {
#endif

	DAQ_API DAQ_STATUS WINAPI DAQ_Open(DAQ_HANDLE *pHandle);
	DAQ_API DAQ_STATUS WINAPI DAQ_SetADCMode(DAQ_HANDLE handle, DAQ_ADCMODE nMode);
	DAQ_API DAQ_STATUS WINAPI DAQ_ReadCalibrator(DAQ_HANDLE handle, DAQ_ADCMODE nMode, unsigned nIndex, double *pGain, double *pOffset);
	DAQ_API DAQ_STATUS WINAPI DAQ_SetADCConf(DAQ_HANDLE handle, unsigned nIndex, unsigned bOn, DAQ_ADCRANGE nRange);
	DAQ_API DAQ_STATUS WINAPI DAQ_SetADCRate(DAQ_HANDLE handle, unsigned long nRate);
	DAQ_API DAQ_STATUS WINAPI DAQ_SetCnterConf(DAQ_HANDLE handle, unsigned nIndex, unsigned bOn);

	DAQ_API DAQ_STATUS WINAPI DAQ_ReadADC(DAQ_HANDLE handle, unsigned nIndex, double *pVolt);
	DAQ_API DAQ_STATUS WINAPI DAQ_ReadADCEx(DAQ_HANDLE handle, unsigned nIndex, unsigned bTrigger, unsigned long nCnt, 
											unsigned long nItvl, unsigned long *pCnt, double *pVolts);
	DAQ_API DAQ_STATUS WINAPI DAQ_ReadADCs(DAQ_HANDLE handle, unsigned *pIndexes, double *pVolts);
	DAQ_API DAQ_STATUS WINAPI DAQ_ReadDgt(DAQ_HANDLE handle, unsigned nIndex, unsigned *pStatus);
	DAQ_API DAQ_STATUS WINAPI DAQ_ReadDgts(DAQ_HANDLE handle, unsigned *pIndexes, unsigned *pStatuses);
	DAQ_API DAQ_STATUS WINAPI DAQ_ReadCnter(DAQ_HANDLE handle, unsigned nIndex, unsigned long *pCounted);
	DAQ_API DAQ_STATUS WINAPI DAQ_ReadCnters(DAQ_HANDLE handle, unsigned *pIndexes, unsigned long *pCounteds);
	DAQ_API DAQ_STATUS WINAPI DAQ_ReadAll(DAQ_HANDLE handle, unsigned *pAIndexes, double *pVolts, 
											unsigned *pDIndexes, unsigned *pStatuses, 
											unsigned *pCIndexes, unsigned long *pCounteds);

	DAQ_API DAQ_STATUS WINAPI DAQ_WriteDACVolt(DAQ_HANDLE handle, unsigned nIndex, unsigned short nValue);
	DAQ_API DAQ_STATUS WINAPI DAQ_WriteDACWave(DAQ_HANDLE handle, unsigned nIndex, unsigned long nItvl, DAQ_DACWAVE nWave);
	DAQ_API DAQ_STATUS WINAPI DAQ_WriteDgtStatus(DAQ_HANDLE handle, unsigned nIndex, unsigned nStatus);
	DAQ_API DAQ_STATUS WINAPI DAQ_WriteDgtPWM(DAQ_HANDLE handle, unsigned nIndex, unsigned long nItvl, unsigned nDuty);
	DAQ_API DAQ_STATUS WINAPI DAQ_Close(DAQ_HANDLE handle);

#ifdef __cplusplus
}
#endif

#endif
