#include "windows.h"
#include "DAQDriver.h"
#include "stdio.h"

/*************************************************************************
 *	this demo project is targeted on MF126, since it supports all the
 *		functionality provided by the dll interfaces.
 ************************************************************************/

#define ANGIN_NUM	14
#define DGTIN_NUM 	5
#define CNTER_NUM 	2
#define BLOCK_SIZE	1000

int main() {
	DAQ_HANDLE handle = NULL;
	unsigned angIndexes[ANGIN_NUM+1], dgtIndexes[DGTIN_NUM+1], cntIndexes[CNTER_NUM+1], statuses[DGTIN_NUM] = {0};
	unsigned long nCounteds[CNTER_NUM] = {0};
	double volts[ANGIN_NUM] = {0};
	double blockVolts[BLOCK_SIZE] = {0};
	double gains[ANGIN_NUM] = {1}, offsets[ANGIN_NUM] = {0};
	unsigned nChlId = 5;	// zero-based channel index
	unsigned bOn = 1;
	unsigned nADCRate = 100;
	double volt = 0;
	unsigned nStatus = 0;
	int i;
	unsigned long nCounted = 0;
	unsigned nDACValue = 0xffff;	// max voltage, for MF126 it's 2.5V
	unsigned long nItvl = 1000;	// in micro seconds
	unsigned nDutyRatio = 50; // 50%
	unsigned bTrigger = 0;	// MF126 doesn't support trigger
	unsigned long nCount = 0;
	if (DAQ_OK == DAQ_Open(&handle)) {
		printf("DAQ_Open success\n\n");
	
		if (DAQ_OK == DAQ_SetADCMode(handle, MODE_SINGLEENDED)) {
			printf("DAQ_SetADCMode success\n");
		} else {
			printf("DAQ_SetADCMode failure\n");
		}

		for (i = 0; i < ANGIN_NUM; ++i) {
			if (DAQ_OK == DAQ_ReadCalibrator(handle, MODE_SINGLEENDED, i, &gains[i], &offsets[i])) {
				printf("DAQ_ReadCalibrator success, ADC channel %u, gain: %f, offset: %f.\n\n", i+1, gains[i], offsets[i]);
			} else {
				printf("DAQ_ReadCalibrator failure\n\n");
			}
		}
		
		// all ADC channels must be turned on and with valid ranges
		// before you can read from them
		if (DAQ_OK == DAQ_SetADCConf(handle, nChlId, bOn, RANGE_N10000_P10000)) {
			printf("DAQ_SetADCConf success, now ADC channel %u is on.\n\n", nChlId+1);
		} else {
			printf("DAQ_SetADCConf failure\n\n");
		}
		
		nChlId = 7;
		bOn = 1;
		if (DAQ_OK == DAQ_SetADCConf(handle, nChlId, bOn, RANGE_N10000_P10000)) {
			printf("DAQ_SetADCConf success, now ADC channel %u is on.\n\n", nChlId+1);
		} else {
			printf("DAQ_SetADCConf failure\n\n");
		}

		if (DAQ_OK == DAQ_SetADCRate(handle, nADCRate)) {
			printf("DAQ_SetADCRate success, ADC sampling rate is %u \n\n", nADCRate);
		} else {
			printf("DAQ_SetADCRate failure\n\n");
		}
		
		nChlId = 7;
		if (DAQ_OK == DAQ_ReadADC(handle, nChlId, &volt)) {
			printf("DAQ_ReadADC success, voltage of ADC channel %u is: %f in volts\n\n", nChlId+1, gains[nChlId]*volt + offsets[nChlId]);
		} else {
			printf("DAQ_ReadADC failure\n\n");
		}

		nChlId = 0;
		// digit input channels are always on, so you can read from them with no configuration
		if (DAQ_OK == DAQ_ReadDgt(handle, nChlId, &nStatus)) {
			printf("DAQ_ReadDgt success, status of digit input channel %u is: %u \n\n", nChlId+1, nStatus);
		} else {
			printf("DAQ_ReadDgt failure\n\n");
		}
		
		if (DAQ_OK == DAQ_ReadDgts(handle, dgtIndexes, statuses)) {
			printf("DAQ_ReadDgts success\n");
			for (i = 0; i < dgtIndexes[0]; ++i) {
				printf("status of digit input channel %u is: %u\n", 1+dgtIndexes[i+1], statuses[i]);
			}
			printf("\n\n");
		} else {
			printf("DAQ_ReadDgts failure\n\n");
		}
		
		nChlId = 0;
		bOn = 1;
		// all counters must be turned on before you can read from them
		// by setting the counter on, the number of counted failing edges
		// is reset to zero
		if (DAQ_OK == DAQ_SetCnterConf(handle, nChlId, bOn)) {
			printf("DAQ_SetCnterConf success, now counter %u is on\n\n", nChlId+1);
		} else {
			printf("DAQ_SetCnterConf failure\n\n");
		}
		
		nChlId = 1;
		bOn = 1;
		if (DAQ_OK == DAQ_SetCnterConf(handle, nChlId, bOn)) {
			printf("DAQ_SetCnterConf success, now counter %u  is on\n\n", nChlId+1);
		} else {
			printf("DAQ_SetCnterConf failure\n\n");
		}

		nChlId = 0;
		if (DAQ_OK == DAQ_ReadCnter(handle, nChlId, &nCounted)) {
			printf("DAQ_ReadCnter success, counter %u counted %u failing edges\n\n", nChlId+1, nCounted);
		} else {
			printf("DAQ_ReadCnter failure\n\n");
		}

		if (DAQ_OK == DAQ_ReadCnters(handle, cntIndexes, nCounteds)) {
			printf("DAQ_ReadCnters success\n");
			for (i = 0; i < cntIndexes[0]; ++i) {
				printf("number of failing edges counted by counter %u is: %u\n", 1+cntIndexes[i+1], nCounteds[i]);
			}
			printf("\n");
		} else {
			printf("DAQ_ReadCnters failure\n\n");
		}
		
		if (DAQ_OK == DAQ_ReadAll(handle, angIndexes, volts, dgtIndexes, statuses, cntIndexes, nCounteds)) {
			printf("DAQ_ReadAll success\n");
			for (i = 0; i < angIndexes[0]; ++i) {
				nChlId = angIndexes[i+1];
				printf("voltage of ADC channel %u is %f in volts\n", 1+nChlId, gains[nChlId]*volts[i] + offsets[nChlId]);
			}
			printf("\n");
			for (i = 0; i < dgtIndexes[0]; ++i) {
				printf("status of digit input channel %u is %u\n", 1+dgtIndexes[i+1], statuses[i]);
			}
			printf("\n");
			for (i = 0; i < cntIndexes[0]; ++i) {
				printf("number of failing edges counted by counter %u is: %u\n", 1+cntIndexes[i+1], nCounteds[i]);
			}
			printf("\n");
		} else {
			printf("DAQ_ReadAll failure\n\n");
		}
		
		if (DAQ_OK == DAQ_ReadADCs(handle, angIndexes, volts)) {
			printf("DAQ_ReadADCs success\n");
			for (i = 0; i < angIndexes[0]; ++i) {
				nChlId = angIndexes[i+1];
				printf("voltage of ADC channel %u is: %f in volts\n", 1+nChlId, gains[nChlId]*volts[i] + offsets[nChlId]);
			}
			printf("\n");
		} else {
			printf("DAQ_ReadADCs failure\n\n");
		}
		
		nChlId = 0;
		if (DAQ_OK == DAQ_WriteDACVolt(handle, nChlId, nDACValue)) {
			printf("DAQ_WriteDACVolt success, DAC channel 1 max voltage\n\n");
		} else {
			printf("DAQ_WriteDACVolt failure\n\n");
		}
		
		nChlId = 1;
		if (DAQ_OK == DAQ_WriteDACWave(handle, nChlId, nItvl, TRIANGLE_WAVE)) {
			printf("DAQ_WriteDACWave success, DAC channel %u triangle waveform, period %u micro seconds\n\n", nChlId+1, nItvl);
		} else {
			printf("DAQ_WriteDACWave failure\n\n");
		}

		nChlId = 1;
		nStatus = 0;
		if (DAQ_OK == DAQ_WriteDgtStatus(handle, nChlId, nStatus)) {
			printf("DAQ_WriteDgtStatus success, digit output channel %u is %s\n\n", nChlId+1, (nStatus ? "high" : "low"));
		} else {
			printf("DAQ_WriteDgtStatus failure\n\n");
		}

		nChlId = 0;
		nItvl = 1;
		if (DAQ_OK == DAQ_WriteDgtPWM(handle, nChlId, nItvl, nDutyRatio)) {
			printf("DAQ_WriteDgtPWM success, digit output channel %u PWM waveform, period %u micro seconds, duty ratio %u%%\n\n", nChlId+1, nItvl, nDutyRatio);
		} else {
			printf("DAQ_WriteDgtPWM failure\n\n");
		}

		// read BLOCK_SIZE data points from ADC channel 8 immediately(don't wait any trigger)
		// with the interval of points nItvl micro seconds
		nChlId = 7;
		nItvl = 1000;
		if (DAQ_OK == DAQ_ReadADCEx(handle, nChlId, bTrigger, BLOCK_SIZE, nItvl, &nCount, blockVolts)) {
			printf("DAQ_ReadADCEx success\n");
			for (i = 0; i < nCount; ++i) {
				printf("blockVolts[%d]: %f\t", i, gains[nChlId]*blockVolts[i] + offsets[nChlId]);
			}
			printf("\n");
		} else {
			printf("DAQ_ReadADCEx failure\n\n");
		}
		
		if (DAQ_OK == DAQ_Close(handle)) {
			printf("close success\n\n");
		} else {
			printf("close failure\n\n");
		}
	} else {
		printf("open failure\n\n");
	}
	return 0;
}